#!/bin/sh
LIST_FILE=/mnt/base/etc/.rsyncSchedule.list
PREFER_SETTING_FILE=/mnt/base/etc/.rsyncPreSetting
WORKNAME=$1
CHECKLOG_RETURN_VALUE=0
WORKNAME_PID=/var/run/.workname.pid
WORK_LOCK=/opt/var/lock/
WORK_LOG_BASE=/etc/rsyncd

Check_log()
{
	#check rsync backup log
	CONTENT=`cat $WORK_LOG_BASE/.$WORKNAME.log | grep "rsync error:"`
	if [ -z "$CONTENT" ];then
		CHECKLOG_RETURN_VALUE=0
	else
		CHECKLOG_RETURN_VALUE=1
	fi
}

Workname_pid()
{
	if [ ! -f $WORKNAME_PID ];then
		touch $WORKNAME_PID
	fi		
	
	#Update pid
	RESULT=`grep -w "$WORKNAME" $WORKNAME_PID`
	if [ ! -z "$RESULT" ];then
		sed -i "/^$WORKNAME/d" $WORKNAME_PID
	fi
	echo "$WORKNAME:$1" >> $WORKNAME_PID
}

if [ -z $WORKNAME ]; then
	echo -e "Usage: $0 [workname] [start|stop|status]\n"
	exit 1
fi

case $2 in
start)
	if [ ! -f $LIST_FILE ] ;then
		echo "can't found $LIST_FILE"
		exit 2
	fi

        [ ! -d $WORK_LOG_BASE ] && mkdir -p $WORK_LOG_BASE  
        [ ! -d $WORK_LOCK ] && mkdir -p $WORK_LOCK 
	
	if [ -f $PREFER_SETTING_FILE ];then
		. $PREFER_SETTING_FILE
	else
		echo -e "PAUSE_TIME=600\nRETRY_TIMES=3\nTIME_INTERVAL=5" > $$PREFER_SETTING_FILE
	fi
	
	#'0' or '1',whether pause other network services(samba,ftp).
	NETWORK_MONOPOLIZE=`cat $LIST_FILE | grep -w -A 10 "\[$WORKNAME\]" | grep  NETWORK_MONOPOLIZE | awk -F'=' '{print $2}'`
	
	#Two difference ACTION,then one adding "--timeout" option
	if [ -z $PAUSE_TIME ];then
		ACTION=`cat $LIST_FILE | grep -w -A 10 "\[$WORKNAME\]" | grep ACTION | awk -F'ACTION=' '{print $2}'`
	else
		STRING1=`cat $LIST_FILE | grep -w -A 10 "\[$WORKNAME\]" | grep ACTION | awk -F'--log-file' '{print $1}'`
		STRING2=`cat $LIST_FILE | grep -w -A 10 "\[$WORKNAME\]" | grep ACTION | awk -F'--log-file' '{print $2}'`
		ACTION="${STRING1#*=} --timeout=$PAUSE_TIME --log-file$STRING2"
		echo "${ACTION}"
	fi
	if [ -z "$ACTION" ];then
		echo "No workname found."
		exit 1
	fi
	
	AUTH_PASSWD=`cat $LIST_FILE | grep -A 10 -w "\[$WORKNAME\]" | grep AUTH_PASSWD`
        export RSYNC_PASSWORD=`echo ${AUTH_PASSWD#*=} | base64 -d`
	echo RSYNC_PASSWORD:$RSYNC_PASSWORD
		
	#Clean log.
	cat /dev/null > $WORK_LOG_BASE/.$WORKNAME.log
	
	#Stop services before backup.
	if [ $NETWORK_MONOPOLIZE -gt 0 ]; then
		echo "Stopping other network service..."
		service proftpd stop
		service samba stop
		service netatalk stop
	fi
	
	#Start backup.
	echo "Starting backup..."
	eval "${ACTION}" &
	Workname_pid $!
	wait
			
	#################Prefer settings on web page################
	
	#CHECKLOG_RETURN_VALUE:0 is success and 1 is failed
	Check_log
	echo CHECKLOG_RETURN_VALUE:$CHECKLOG_RETURN_VALUE
	
	#If backup successfully,then enable serivce.
	if [ $CHECKLOG_RETURN_VALUE -eq 0 ];then
		if [ $NETWORK_MONOPOLIZE -gt 0 ]; then
			service samba stop
			service proftpd stop
		fi
		exit 0
	fi
	
	#If backup failed,then retry.
	times=$RETRY_TIMES
	time_interval=$TIME_INTERVAL
	RETRY=0
	
	[ "x$times" = "x" ] && exit 0
	
	while [ $times -gt 0 ]
		do
			echo -n "Sleeping $time_interval seconds:"
			sleep $time_interval
			echo "waked."
			times=`expr $times - 1`	
			RETRY=`expr $RETRY + 1`
			echo RETRY_TIMES=$RETRY > $WORK_LOG_BASE/.$WORKNAME.log 
		
			echo "Starting backup..."
			eval "${ACTION}" &
			Put_pid $!
			wait
		
			Check_log
			echo CHECKLOG_RETURN_VALUE:$CHECKLOG_RETURN_VALUE
			[ $CHECKLOG_RETURN_VALUE -eq 0 ] && break
	
			#The following design for web UI
			if [ $RETRY -eq $RETRY_TIMES ];then
				sed -i "/^RETRY_TIMES=/c\\RETRY_TIMES=`expr $RETRY_TIMES + 1`" $WORK_LOG_BASE/.$WORKNAME.log
			fi
		done
	###########################################################
	
	if [ $NETWORK_MONOPOLIZE -gt 0 ]; then
		echo "Starting other network service..."
		service proftpd start &
		service samba start &
		service netatalk start &
	fi
	;;
stop)
	#Stop script.
	echo "Stopping rsyncBackup.sh $WORKNAME start..."
	result=`ps aux | grep -v "grep" | grep "rsyncBackup.sh $WORKNAME start"`
	if [ ! -z "$result" ];then
		echo $result | awk '{print $1}' | while read LINE
		do
			echo $LINE
			pid=$LINE
			kill -9 $pid
	done
	fi
	
	#Stop rsync backup program
	PID=`cat $WORKNAME_PID | grep -w $WORKNAME | awk -F':' '{print $2}'`
	echo "Stopping backup program[$PID]"
	if [ ! -z "$PID" ];then
		result=`ps aux | grep -v "grep" | grep -w $PID`
		[ ! -z "$result" ] && kill -9 $PID
	fi

	#Put error message
	echo -e "rsync error:killed" > $WORK_LOG_BASE/.$WORKNAME.log
	;;
status)
	PID=`cat $WORKNAME_PID | grep -w $WORKNAME | awk -F':' '{print $2}'`
	if [ ! -z "$PID" ];then
		result=`ps | grep -v "grep" | grep -w $PID`
		if [ -z "$result" ];then
			echo "stopped"
		else
			echo "running"
		fi
	else
		echo "stopped"
	fi
	;;
*)
	echo -e "Usage: $0 [workname] [start|stop|status]\n"
	;;
esac
exit 0
